@@ -4,15 +4,20 @@ from unittest import mock
 
 from django.db import connection, transaction
 from django.db.models import (
+    BooleanField,
     Case,
     Count,
     DecimalField,
+    ExpressionWrapper,
     F,
     FilteredRelation,
     Q,
     Sum,
+    Value,
     When,
 )
+from django.db.models.functions import Concat
+from django.db.models.lookups import Exact, IStartsWith
 from django.test import TestCase
 from django.test.testcases import skipUnlessDBFeature
 
@@ -211,19 +216,6 @@ class FilteredRelationTests(TestCase):
             str(queryset.query),
         )
 
-    def test_period_forbidden(self):
-        msg = (
-            "FilteredRelation doesn't support aliases with periods (got 'book.alice')."
-        )
-        with self.assertRaisesMessage(ValueError, msg):
-            Author.objects.annotate(
-                **{
-                    "book.alice": FilteredRelation(
-                        "book", condition=Q(book__title__iexact="poem by alice")
-                    )
-                }
-            )
-
     def test_multiple(self):
         qs = (
             Author.objects.annotate(
@@ -590,6 +582,28 @@ class FilteredRelationTests(TestCase):
             lambda x: (x.author, x.book_title, x.preferred_by_author_pk),
         )
 
+    def test_three_level_nested_chained_relations(self):
+        borrower = Borrower.objects.create(name="Jenny")
+        Reservation.objects.create(
+            borrower=borrower,
+            book=self.book1,
+            state=Reservation.STOPPED,
+        )
+        qs = Author.objects.annotate(
+            my_books=FilteredRelation("book"),
+            my_reserved_books=FilteredRelation(
+                "my_books__reservation",
+                condition=Q(my_books__reservation__state=Reservation.STOPPED),
+            ),
+            my_readers=FilteredRelation(
+                "my_reserved_books__borrower",
+                condition=Q(my_reserved_books__borrower=borrower),
+            ),
+        )
+        self.assertSequenceEqual(
+            qs.filter(my_readers=borrower).values_list("name", flat=True), ["Alice"]
+        )
+
     def test_deep_nested_foreign_key(self):
         qs = (
             Book.objects.annotate(
@@ -642,6 +656,26 @@ class FilteredRelationTests(TestCase):
                 ),
             )
 
+    def test_condition_with_exact_lookup_outside_relation_name(self):
+        qs = Author.objects.annotate(
+            book_editor=FilteredRelation(
+                "book__editor",
+                condition=Q(book__author__name="book"),
+            ),
+        ).filter(book_editor__isnull=True)
+        self.assertEqual(qs.count(), 4)
+
+    def test_condition_with_func_and_lookup_outside_relation_name(self):
+        qs = Author.objects.annotate(
+            book_editor=FilteredRelation(
+                "book__editor",
+                condition=Q(
+                    book__title=Concat(Value("The book by "), F("book__author__name"))
+                ),
+            ),
+        ).filter(book_editor__isnull=False)
+        self.assertEqual(qs.count(), 1)
+
     def test_condition_deeper_relation_name(self):
         msg = (
             "FilteredRelation's condition doesn't support nested relations "
@@ -656,6 +690,19 @@ class FilteredRelationTests(TestCase):
                 ),
             )
 
+    def test_condition_deeper_relation_name_implicit_exact(self):
+        msg = (
+            "FilteredRelation's condition doesn't support nested relations "
+            "deeper than the relation_name (got 'book__editor__name' for 'book')."
+        )
+        with self.assertRaisesMessage(ValueError, msg):
+            Author.objects.annotate(
+                book_editor=FilteredRelation(
+                    "book",
+                    condition=Q(book__editor__name="b"),
+                ),
+            )
+
     def test_with_empty_relation_name_error(self):
         with self.assertRaisesMessage(ValueError, "relation_name cannot be empty."):
             FilteredRelation("", condition=Q(blank=""))
@@ -698,6 +745,134 @@ class FilteredRelationTests(TestCase):
             FilteredRelation("book", condition=Q(book__title="b")), mock.ANY
         )
 
+    def test_conditional_expression(self):
+        qs = Author.objects.annotate(
+            the_book=FilteredRelation("book", condition=Q(Value(False))),
+        ).filter(the_book__isnull=False)
+        self.assertSequenceEqual(qs, [])
+
+    def test_expression_outside_relation_name(self):
+        qs = Author.objects.annotate(
+            book_editor=FilteredRelation(
+                "book__editor",
+                condition=Q(
+                    Exact(F("book__author__name"), "Alice"),
+                    Value(True),
+                    book__title__startswith="Poem",
+                ),
+            ),
+        ).filter(book_editor__isnull=False)
+        self.assertSequenceEqual(qs, [self.author1])
+
+    def test_conditional_expression_with_case(self):
+        qs = Book.objects.annotate(
+            alice_author=FilteredRelation(
+                "author",
+                condition=Q(
+                    Case(When(author__name="Alice", then=True), default=False),
+                ),
+            ),
+        ).filter(alice_author__isnull=False)
+        self.assertCountEqual(qs, [self.book1, self.book4])
+
+    def test_conditional_expression_outside_relation_name(self):
+        tests = [
+            Q(Case(When(book__author__name="Alice", then=True), default=False)),
+            Q(
+                ExpressionWrapper(
+                    Q(Value(True), Exact(F("book__author__name"), "Alice")),
+                    output_field=BooleanField(),
+                ),
+            ),
+        ]
+        for condition in tests:
+            with self.subTest(condition=condition):
+                qs = Author.objects.annotate(
+                    book_editor=FilteredRelation("book__editor", condition=condition),
+                ).filter(book_editor__isnull=True)
+                self.assertSequenceEqual(qs, [self.author2, self.author2])
+
+    def test_conditional_expression_with_lookup(self):
+        lookups = [
+            Q(book__title__istartswith="poem"),
+            Q(IStartsWith(F("book__title"), "poem")),
+        ]
+        for condition in lookups:
+            with self.subTest(condition=condition):
+                qs = Author.objects.annotate(
+                    poem_book=FilteredRelation("book", condition=condition)
+                ).filter(poem_book__isnull=False)
+                self.assertSequenceEqual(qs, [self.author1])
+
+    def test_conditional_expression_with_expressionwrapper(self):
+        qs = Author.objects.annotate(
+            poem_book=FilteredRelation(
+                "book",
+                condition=Q(
+                    ExpressionWrapper(
+                        Q(Exact(F("book__title"), "Poem by Alice")),
+                        output_field=BooleanField(),
+                    ),
+                ),
+            ),
+        ).filter(poem_book__isnull=False)
+        self.assertSequenceEqual(qs, [self.author1])
+
+    def test_conditional_expression_with_multiple_fields(self):
+        qs = Author.objects.annotate(
+            my_books=FilteredRelation(
+                "book__author",
+                condition=Q(Exact(F("book__author__name"), F("book__author__name"))),
+            ),
+        ).filter(my_books__isnull=True)
+        self.assertSequenceEqual(qs, [])
+
+    def test_conditional_expression_rhs_contains_relation_name(self):
+        qs = Book.objects.annotate(
+            rel=FilteredRelation(
+                "editor",
+                condition=Q(id=1 * F("number_editor")),
+            )
+        ).filter(rel__isnull=True)
+        self.assertSequenceEqual(qs, [])
+
+    def test_conditional_expression_rhs_startswith_relation_name(self):
+        qs = Book.objects.annotate(
+            rel=FilteredRelation(
+                "editor",
+                condition=Q(id=1 * F("editor_number")),
+            )
+        ).filter(rel__isnull=True)
+        self.assertSequenceEqual(qs, [])
+
+    def test_conditional_expression_lhs_startswith_relation_name(self):
+        qs = Book.objects.annotate(
+            rel=FilteredRelation(
+                "editor",
+                condition=Q(editor_number__gt=1),
+            )
+        ).filter(rel__isnull=True)
+        self.assertSequenceEqual(qs, [])
+
+    def test_conditional_expression_lhs_contains_relation_name(self):
+        qs = Book.objects.annotate(
+            rel=FilteredRelation(
+                "editor",
+                condition=Q(number_editor__gt=1),
+            )
+        ).filter(rel__isnull=True)
+        self.assertSequenceEqual(qs, [])
+
+    def test_conditional_expression_does_not_support_queryset(self):
+        msg = "Passing a QuerySet within a FilteredRelation is not supported."
+        with self.assertRaisesMessage(ValueError, msg):
+            Author.objects.annotate(
+                poem_book=FilteredRelation(
+                    "book",
+                    condition=Q(book__in=Book.objects.filter(title__istartswith="a")),
+                ),
+            ).filter(poem_book__isnull=False)
+
 
 class FilteredRelationAggregationTests(TestCase):
     @classmethod
@@ -806,6 +981,47 @@ class FilteredRelationAggregationTests(TestCase):
             qs.annotate(total=Count("pk")).values("total"), [{"total": 1}]
         )
 
+    def test_condition_spans_join(self):
+        self.assertSequenceEqual(
+            Book.objects.annotate(
+                contains_editor_author=FilteredRelation(
+                    "author", condition=Q(author__name__icontains=F("editor__name"))
+                )
+            ).filter(
+                contains_editor_author__isnull=False,
+            ),
+            [self.book1],
+        )
+
+    def test_condition_spans_join_chained(self):
+        self.assertSequenceEqual(
+            Book.objects.annotate(
+                contains_editor_author=FilteredRelation(
+                    "author", condition=Q(author__name__icontains=F("editor__name"))
+                ),
+                contains_editor_author_ref=FilteredRelation(
+                    "author",
+                    condition=Q(author__name=F("contains_editor_author__name")),
+                ),
+            ).filter(
+                contains_editor_author_ref__isnull=False,
+            ),
+            [self.book1],
+        )
+
+    def test_condition_self_ref(self):
+        self.assertSequenceEqual(
+            Book.objects.annotate(
+                contains_author=FilteredRelation(
+                    "author",
+                    condition=Q(title__icontains=F("author__name")),
+                )
+            ).filter(
+                contains_author__isnull=False,
+            ),
+            [self.book1],
+        )
+
 
 class FilteredRelationAnalyticalAggregationTests(TestCase):
     @classmethod
